@@ -48,8 +48,9 @@ logger = Logger(__name__)
 
 
 def main() -> None:
-    # Check environment and versions if not in the cloud
-    if "LIGHTNING_APP_STATE_URL" not in os.environ:
+    # Check environment and versions if not in the cloud and not testing
+    is_testing = bool(int(os.getenv("LIGHTING_TESTING", "0")))
+    if not is_testing and "LIGHTNING_APP_STATE_URL" not in os.environ:
         # Enforce running in PATH Python
         _check_environment_and_redirect()
 
@@ -75,8 +76,6 @@ def main() -> None:
                 else:
                     message = f"You are connected to the cloud Lightning App: {app_name}."
 
-                click.echo(" ")
-
                 if (len(sys.argv) > 1 and sys.argv[1] in ["-h", "--help"]) or len(sys.argv) == 1:
                     _list_app_commands()
                 else:
@@ -205,7 +204,7 @@ def login() -> None:
     auth.clear()
 
     try:
-        auth._run_server()
+        auth.authenticate()
     except ConnectionError:
         click.echo(f"Unable to connect to {get_lightning_cloud_url()}. Please check your internet connection.")
         exit(1)
