@@ -8,8 +8,6 @@ from base64 import b64encode
 from itertools import cycle
 from typing import Any, Dict, List, Tuple, Type
 
-import aiohttp
-import aiohttp.client_exceptions
 import requests
 import uvicorn
 from fastapi import Depends, FastAPI, HTTPException, Request
@@ -22,8 +20,13 @@ from starlette.status import HTTP_401_UNAUTHORIZED
 from lightning_app.core.flow import LightningFlow
 from lightning_app.core.work import LightningWork
 from lightning_app.utilities.app_helpers import Logger
+from lightning_app.utilities.imports import _is_aiohttp_available, requires
 from lightning_app.utilities.packaging.cloud_compute import CloudCompute
 
+if _is_aiohttp_available():
+    import aiohttp
+    import aiohttp.client_exceptions
+
 logger = Logger(__name__)
 
 
@@ -114,6 +117,7 @@ class _LoadBalancer(LightningWork):
         \**kwargs: Arguments passed to :func:`LightningWork.init` like ``CloudCompute``, ``BuildConfig``, etc.
     """
 
+    @requires(["aiohttp"])
     def __init__(
         self,
         input_type: BaseModel,
@@ -445,8 +449,16 @@ class AutoScaler(LightningFlow):
 
     def create_work(self) -> LightningWork:
         """Replicates a LightningWork instance with args and kwargs provided via ``__init__``."""
-        # TODO: Remove `start_with_flow=False` for faster initialization on the cloud
-        return self._work_cls(*self._work_args, **self._work_kwargs, start_with_flow=False)
+        cloud_compute = self._work_kwargs.get("cloud_compute", None)
+        self._work_kwargs.update(
+            dict(
+                # TODO: Remove `start_with_flow=False` for faster initialization on the cloud
+                start_with_flow=False,
+                # don't try to create multiple works in a single machine
+                cloud_compute=cloud_compute.clone() if cloud_compute else None,
+            )
+        )
+        return self._work_cls(*self._work_args, **self._work_kwargs)
 
     def add_work(self, work) -> str:
         """Adds a new LightningWork instance.
