@@ -3,10 +3,10 @@ from dataclasses import dataclass
 
 
 @dataclass
-class Config:
+class _Config:
     id = os.getenv("LIGHTNING_USER_ID")
     key = os.getenv("LIGHTNING_API_KEY")
-    url = os.getenv("LIGHTNING_CLOUD_URL", "")
+    url = os.getenv("LIGHTNING_CLOUD_URL", "https://lightning.ai")
     api_key = os.getenv("LIGHTNING_API_KEY")
     username = os.getenv("LIGHTNING_USERNAME")
     video_location = os.getenv("VIDEO_LOCATION", "./artifacts/videos")
