@@ -4,21 +4,27 @@ import os
 import os.path as osp
 import shutil
 import sys
+import traceback
+from dataclasses import asdict
 from getpass import getuser
 from importlib.util import module_from_spec, spec_from_file_location
 from tempfile import gettempdir
 from typing import Any, Callable, Dict, List, Optional, Union
 
 import requests
+from fastapi import HTTPException
 from pydantic import BaseModel
 
 from lightning_app.api.http_methods import Post
-from lightning_app.api.request_types import APIRequest, CommandRequest
-from lightning_app.utilities.app_helpers import is_overridden
+from lightning_app.api.request_types import _APIRequest, _CommandRequest, _RequestResponse
+from lightning_app.utilities import frontend
+from lightning_app.utilities.app_helpers import is_overridden, Logger
 from lightning_app.utilities.cloud import _get_project
 from lightning_app.utilities.network import LightningClient
 from lightning_app.utilities.state import AppState
 
+logger = Logger(__name__)
+
 
 def makedirs(path: str):
     r"""Recursive directory creation function."""
@@ -31,13 +37,15 @@ def makedirs(path: str):
 
 class ClientCommand:
 
-    DESCRIPTION = ""
+    description: str = ""
+    requirements: List[str] = []
 
-    def __init__(self, method: Callable, requirements: Optional[List[str]] = None) -> None:
+    def __init__(self, method: Callable):
         self.method = method
-        flow = getattr(method, "__self__", None)
+        if not self.description:
+            self.description = self.method.__doc__ or ""
+        flow = getattr(self.method, "__self__", None)
         self.owner = flow.name if flow else None
-        self.requirements = requirements
         self.models: Optional[Dict[str, BaseModel]] = None
         self.app_url = None
         self._state = None
@@ -60,10 +68,17 @@ class ClientCommand:
     def run(self, **cli_kwargs) -> None:
         """Overrides with the logic to execute on the client side."""
 
-    def invoke_handler(self, config: BaseModel) -> Dict[str, Any]:
+    def invoke_handler(self, config: Optional[BaseModel] = None) -> Dict[str, Any]:
         command = self.command_name.replace(" ", "_")
-        resp = requests.post(self.app_url + f"/command/{command}", data=config.json())
-        assert resp.status_code == 200, resp.json()
+        resp = requests.post(self.app_url + f"/command/{command}", data=config.json() if config else None)
+        if resp.status_code != 200:
+            try:
+                detail = str(resp.json())
+            except Exception:
+                detail = "Internal Server Error"
+            print(f"Failed with status code {resp.status_code}. Detail: {detail}")
+            sys.exit(0)
+
         return resp.json()
 
     def _to_dict(self):
@@ -95,7 +110,9 @@ def _download_command(
             if not os.path.exists(target_file):
                 client = LightningClient()
                 project_id = _get_project(client).project_id
-                response = client.lightningapp_instance_service_list_lightningapp_instance_artifacts(project_id, app_id)
+                response = client.lightningapp_instance_service_list_lightningapp_instance_artifacts(
+                    project_id=project_id, id=app_id
+                )
                 for artifact in response.artifacts:
                     if f"commands/{command_name}.py" == artifact.filename:
                         resp = requests.get(artifact.url, allow_redirects=True)
@@ -109,7 +126,7 @@ def _download_command(
     mod = module_from_spec(spec)
     sys.modules[cls_name] = mod
     spec.loader.exec_module(mod)
-    command = getattr(mod, cls_name)(method=None, requirements=[])
+    command = getattr(mod, cls_name)(method=None)
     if tmpdir and os.path.exists(tmpdir):
         shutil.rmtree(tmpdir)
     return command
@@ -153,19 +170,21 @@ def _validate_client_command(command: ClientCommand):
 
 
 def _upload_command(command_name: str, command: ClientCommand) -> Optional[str]:
-    from lightning_app.storage.path import _is_s3fs_available, filesystem, shared_storage_path
+    from lightning_app.storage.path import _filesystem, _is_s3fs_available, _shared_storage_path
 
+    command_name = command_name.replace(" ", "_")
     filepath = f"commands/{command_name}.py"
-    remote_url = str(shared_storage_path() / "artifacts" / filepath)
-    fs = filesystem()
+    remote_url = str(_shared_storage_path() / "artifacts" / filepath)
+    fs = _filesystem()
 
     if _is_s3fs_available():
         from s3fs import S3FileSystem
 
         if not isinstance(fs, S3FileSystem):
             return
+
         source_file = str(inspect.getfile(command.__class__))
-        remote_url = str(shared_storage_path() / "artifacts" / filepath)
+        remote_url = str(_shared_storage_path() / "artifacts" / filepath)
         fs.put(source_file, remote_url)
         return filepath
 
@@ -186,42 +205,74 @@ def _prepare_commands(app) -> List:
     return commands
 
 
-def _process_api_request(app, request: APIRequest) -> None:
+def _process_api_request(app, request: _APIRequest):
     flow = app.get_component_by_name(request.name)
     method = getattr(flow, request.method_name)
-    response = method(*request.args, **request.kwargs)
-    app.api_response_queue.put({"response": response, "id": request.id})
+    try:
+        response = _RequestResponse(content=method(*request.args, **request.kwargs), status_code=200)
+    except HTTPException as e:
+        logger.error(repr(e))
+        response = _RequestResponse(status_code=e.status_code, content=e.detail)
+    except Exception:
+        logger.error(traceback.print_exc())
+        response = _RequestResponse(status_code=500)
+    return {"response": response, "id": request.id}
 
 
-def _process_command_requests(app, request: CommandRequest) -> None:
+def _process_command_requests(app, request: _CommandRequest):
     for command in app.commands:
         for command_name, method in command.items():
             command_name = command_name.replace(" ", "_")
             if request.method_name == command_name:
                 # 2.1: Evaluate the method associated to a specific command.
                 # Validation is done on the CLI side.
-                response = method(*request.args, **request.kwargs)
-                app.api_response_queue.put({"response": response, "id": request.id})
+                try:
+                    response = _RequestResponse(content=method(*request.args, **request.kwargs), status_code=200)
+                except HTTPException as e:
+                    logger.error(repr(e))
+                    response = _RequestResponse(status_code=e.status_code, content=e.detail)
+                except Exception:
+                    logger.error(traceback.print_exc())
+                    response = _RequestResponse(status_code=500)
+                return {"response": response, "id": request.id}
 
 
-def _process_requests(app, request: Union[APIRequest, CommandRequest]) -> None:
+def _process_requests(app, requests: List[Union[_APIRequest, _CommandRequest]]) -> None:
     """Convert user commands to API endpoint."""
-    if isinstance(request, APIRequest):
-        _process_api_request(app, request)
-    else:
-        _process_command_requests(app, request)
+    responses = []
+    for request in requests:
+        if isinstance(request, _APIRequest):
+            response = _process_api_request(app, request)
+        else:
+            response = _process_command_requests(app, request)
+
+        if response:
+            responses.append(response)
+
+    app.api_response_queue.put(responses)
 
 
-def _collect_open_api_extras(command) -> Dict:
+def _collect_open_api_extras(command, info) -> Dict:
     if not isinstance(command, ClientCommand):
+        if command.__doc__ is not None:
+            return {"description": command.__doc__}
         return {}
-    return {
+
+    extras = {
         "cls_path": inspect.getfile(command.__class__),
         "cls_name": command.__class__.__name__,
+        "description": command.description,
     }
+    if command.requirements:
+        extras.update({"requirements": command.requirements})
+    if info:
+        extras.update({"app_info": asdict(info)})
+    return extras
 
 
-def _commands_to_api(commands: List[Dict[str, Union[Callable, ClientCommand]]]) -> List:
+def _commands_to_api(
+    commands: List[Dict[str, Union[Callable, ClientCommand]]], info: Optional[frontend.AppInfo] = None
+) -> List:
     """Convert user commands to API endpoint."""
     api = []
     for command in commands:
@@ -233,7 +284,7 @@ def _commands_to_api(commands: List[Dict[str, Union[Callable, ClientCommand]]])
                     v.method if isinstance(v, ClientCommand) else v,
                     method_name=k,
                     tags=["app_client_command"] if isinstance(v, ClientCommand) else ["app_command"],
-                    openapi_extra=_collect_open_api_extras(v),
+                    openapi_extra=_collect_open_api_extras(v, info),
                 )
             )
     return api
