@@ -82,7 +82,7 @@ class CloudCompute:
 
         # All `default` CloudCompute are identified in the same way.
         if self._internal_id is None:
-            self._internal_id = "default" if self.name == "default" else uuid4().hex[:7]
+            self._internal_id = self._generate_id()
 
         # Internal arguments for now.
         self.preemptible = False
@@ -118,6 +118,14 @@ class CloudCompute:
     def is_default(self) -> bool:
         return self.name == "default"
 
+    def _generate_id(self):
+        return "default" if self.name == "default" else uuid4().hex[:7]
+
+    def clone(self):
+        new_dict = self.to_dict()
+        new_dict["_internal_id"] = self._generate_id()
+        return self.from_dict(new_dict)
+
 
 def _verify_mount_root_dirs_are_unique(mounts: Union[None, Mount, List[Mount], Tuple[Mount]]) -> None:
     if isinstance(mounts, (list, tuple, set)):
